@@ -20,6 +20,8 @@ from object_clerk.config import CLIENT_CONFIG
 from object_clerk.config import CONNECTION_RETRY_EXCEPTIONS
 from object_clerk.config import MULTIPART_THRESHOLD
 from object_clerk.config import UPLOAD_CONFIG
+from object_clerk.exceptions import ArchiveCopyNotSupportedException
+from object_clerk.exceptions import ArchiveMoveNotSupportedException
 from object_clerk.exceptions import ObjectSaveException
 from object_clerk.exceptions import ObjectVerificationException
 from object_clerk.utils import mutate_client_exceptions
@@ -29,7 +31,7 @@ logger = logging.getLogger(__name__)
 # Configure boto to not retry; handled by the retry api
 
 
-__all__ = ["ObjectClerk"]
+__all__ = ["ObjectClerk", "ArchiveClerk"]
 
 
 class ObjectClerk:
@@ -513,3 +515,203 @@ class ObjectClerk:
             **self.retry_config,
         )
         return response
+
+
+class ArchiveClerk(ObjectClerk):
+    """
+    The ObjectClerk intended for archiving operations
+
+    Environment configuration:
+    MULTIPART_THRESHOLD : The threshold for multi part uploads.  Uploads and downloads have to use
+      the same threshold for the checksum algorithm to verify integrity.  Default is 8 MB.
+    """
+
+    def __init__(
+        self,
+        host: str,
+        port: int,
+        access_key: str,
+        secret_key: str,
+        retry_delay: int,
+        retry_backoff: int,
+        retry_jitter: Union[int, Tuple[int, int]],
+        retry_max_delay: int,
+        retry_tries: int = -1,
+        use_ssl: bool = False,
+        archive_storage_class: str = None,
+    ):
+        """
+        Initialize the Archive clerk
+        :param host: Host name or ip for an s3 Gateway
+        :param port: Post the s3 gateway listens on
+        :param access_key: Access Key for the gateway
+        :param secret_key: Secret Key for the gateway
+        :param retry_delay: initial delay between attempts for connection errors.
+        :param retry_backoff: multiplier applied to delay between attempts to connect.
+        :param retry_jitter: extra seconds added to delay between attempts to connect.
+                   fixed if a number, random if a range tuple (min, max)
+        :param retry_max_delay: the maximum value of delay between connection attempts.
+        :param retry_tries: Number of time to retry connecting ot hte gateway. -1 for indefinite retries
+        :param use_ssl: True for https and False for http
+        :param archive_storage_class: Archive to AWS Glacier Deep Archive when DEEP_ARCHIVE.  Archive to Ceph otherwise.
+        """
+        super().__init__(
+            host,
+            port,
+            access_key,
+            secret_key,
+            retry_delay,
+            retry_backoff,
+            retry_jitter,
+            retry_max_delay,
+            retry_tries,
+            use_ssl,
+        )
+        if isinstance(archive_storage_class, str):
+            self.archive_storage_class = archive_storage_class.upper()
+        else:
+            self.archive_storage_class = "NOT_PROVIDED"
+
+    def __repr__(self):
+        return (
+            f"ArchiveClerk(host={self.host}, port={self.port}, access_key={self.access_key}, "
+            f"secret_key=<SECRET KEY>, retry_delay={self.retry_delay}, retry_backoff={self.retry_backoff}, "
+            f"retry_jitter={self.retry_jitter}, retry_max_delay={self.retry_max_delay}, "
+            f"retry_tries={self.retry_tries}, use_ssl={self.use_ssl})"
+        )
+
+    def __str__(self):
+        return f"ArchiveClerk connected to {self.endpoint_url} retrying connection errors with {self.retry_config}"
+
+    def copy_object(
+        self,
+        source_bucket: str,
+        source_object_key: str,
+        destination_bucket: str,
+        destination_object_key: str,
+        verify_checksum: bool = True,
+    ) -> None:
+        """
+        Always raise ArchiveCopyNotSupportedException to prevent copy within archive-store
+        :param source_bucket: Bucket of the object being copied
+        :param source_object_key: Object key of the object being copied
+        :param destination_bucket: Bucket to copy to
+        :param destination_object_key: Object key of the copied object
+        :param verify_checksum: Boolean indicator of whether to verify the checksum of the
+        copied file matches the original
+        :return: None
+        :raises: ArchiveCopyNotSupportedException
+        """
+        raise ArchiveCopyNotSupportedException(
+            "Copy not supported for objects in the archive store"
+        )
+
+    def move_object(
+        self,
+        source_bucket: str,
+        source_object_key: str,
+        destination_bucket: str,
+        destination_object_key: str,
+        verify_checksum: bool = True,
+    ) -> None:
+        """
+        Always raise ArchiveMoveNotSupportedException to prevent move within archive-store
+        :param source_bucket: Bucket of the object being copied
+        :param source_object_key: Object key of the object being copied and will be deleted
+        :param destination_bucket: Bucket to copy to
+        :param destination_object_key: Object key of the copied object
+        :param verify_checksum: Boolean indicator of whether to verify the checksum of the
+        copied file matches the original
+        :return: None
+        :raises: ArchiveMoveNotSupportedException
+        """
+        self.copy_object(
+            source_bucket,
+            source_object_key,
+            destination_bucket,
+            destination_object_key,
+            verify_checksum,
+        )
+        raise ArchiveMoveNotSupportedException(
+            "Move not supported for objects in the archive store"
+        )
+
+    @retry(exceptions=ObjectVerificationException, **CHECKSUM_RETRY_CONFIG)
+    def upload_object(
+        self,
+        object_data: Union[str, Path, BufferedReader, BytesIO, bytes],
+        bucket: str,
+        object_key: str,
+        verify_checksum: bool = True,
+        content_type: str = "application/octet-stream",
+        metadata: dict = None,
+    ) -> None:
+        """
+        Uploads a data to the specified bucket and object key retrying connection errors according to instance
+        retry specification and checksum failures according to internal config if requested (default)
+        :param object_data: Data to upload
+        :param bucket: Bucket to upload to
+        :param object_key: Object Key in the bucket for the object after upload
+        :param verify_checksum: Boolean indicator of whether to verify the checksum of the uploaded file matches
+        what is in the S3 bucket when archive_storage_class is not DEEP_ARCHIVE. The checksum is never verified when
+        archive_storage_class is DEEP_ARCHIVE.
+        :param content_type: Content type of the file to be uploaded.  Will be added to the ExtraArgs as Content-Type
+        :param metadata: metadata for the file to be uploaded.  Will be added to the ExtraArgs as Metadata
+        :return: None
+        :raises: ObjectVerificationException, ObjectSaveException, ObjectClerkException, ObjectClerkServerAuthException
+        """
+        metadata = metadata or {}
+        bytes_obj = self._data_to_bytes(object_data)
+        # get checksum before boto operations
+        checksum = self._get_object_checksum(bytes_obj)
+        if checksum == md5(b"").hexdigest():
+            raise ObjectSaveException("Attempt to upload an empty file not allowed.")
+        # proactively retain checksum for all archive objects
+        metadata["dkist-checksum"] = checksum
+        retry_call(
+            self._upload_object,
+            fkwargs={
+                "bytes_obj": bytes_obj,
+                "bucket": bucket,
+                "object_key": object_key,
+                "content_type": content_type,
+                "metadata": metadata,
+            },
+            **self.retry_config,
+        )
+        # GET request will fail for DEEP_ARCHIVE storage class, so skip when archive_storage_class is DEEP_ARCHIVE
+        if verify_checksum and self.archive_storage_class != "DEEP_ARCHIVE":
+            etag = self.get_object_info(bucket, object_key).get("etag")
+            try:
+                self._verify_checksum(checksum, etag)
+            except ObjectVerificationException as e:
+                logger.warning(f"Saved object does not match check sum: detail={e}")
+                self.delete_object(bucket, object_key)
+                logger.debug(f"Saved object is removed: bucket={bucket}, object_key={object_key}")
+                raise e
+
+    @mutate_client_exceptions
+    def _upload_object(
+        self, bytes_obj: bytes, bucket: str, object_key: str, content_type: str, metadata: dict
+    ) -> None:
+        """
+        Executes the boto3.client.upload_fileobj function with ClientError exceptions
+        transformed to more precise ObjectClerk exceptions
+        :param bucket: Bucket to upload the object to
+        :param object_key: Key of the object in the bucket to upload to
+        :param bytes_obj: bytes to upload
+        :param content_type: Content type of the file to be uploaded.  Will be added to the ExtraArgs as Content-Type
+        :param metadata: metadata for the file to be uploaded.  Will be added to the ExtraArgs as Metadata
+        :return: None
+        """
+        logger.debug(f"Attempting object upload: bucket={bucket}, object_key={object_key}")
+        extra_args = {"ContentType": content_type, "Metadata": metadata}
+        if self.archive_storage_class == "DEEP_ARCHIVE":
+            extra_args["StorageClass"] = "DEEP_ARCHIVE"
+        self.client.upload_fileobj(
+            BytesIO(bytes_obj),
+            Bucket=bucket,
+            Key=object_key,
+            Config=UPLOAD_CONFIG,
+            ExtraArgs=extra_args,
+        )
