@@ -39,3 +39,21 @@ class ObjectSaveException(ObjectClerkException):
     """
     Exception when objects cannot be saved due to size or readability
     """
+
+
+class ArchiveClerkException(ObjectClerkException):
+    """
+    Base Exception for the archive clerk
+    """
+
+
+class ArchiveCopyNotSupportedException(ArchiveClerkException):
+    """
+    Exception an attempt to copy an object is made from the archive clerk
+    """
+
+
+class ArchiveMoveNotSupportedException(ArchiveClerkException):
+    """
+    Exception an attempt to move an object is made from the archive clerk
+    """
