@@ -4,6 +4,7 @@ Tests for the ObjectClerk API
 Tests marked @pytest.mark.development require environment variables
 to specify an s3 instance location and tokens
 """
+import os
 from hashlib import md5
 from io import BufferedReader
 from io import BytesIO
@@ -15,11 +16,13 @@ from uuid import uuid4
 import boto3
 import pytest
 
+from object_clerk import ArchiveClerk
 from object_clerk import ObjectClerk
 from object_clerk import ObjectNotFoundException
 from object_clerk import ObjectSaveException
 from object_clerk.config import MULTIPART_THRESHOLD
 
+ARCHIVE_STORE_USE_SSL = environ.get("ARCHIVE_STORE_USE_SSL", "False").lower() == "true"
 
 """
 S3 endpoint configuration
@@ -39,6 +42,16 @@ RETRY = {
 }
 
 
+def get_abs_path(test_filename: str = None) -> str:
+    """
+    Return the absolute path to the test data
+    :param test_filename: name of the test file in current working directory
+    :return: absolute path of the test file
+    """
+    pwd = os.getcwd()
+    return os.path.join(pwd, test_filename)
+
+
 """
 Fixtures for data and other dependencies
 """
@@ -50,7 +63,8 @@ def boto_client() -> boto3.client:
     Configure a boto3.client for s3
     :return: boto3.client instance
     """
-    endpoint_url = f"http://{HOST}:{PORT}/"
+    schema = "https" if ARCHIVE_STORE_USE_SSL or PORT == "443" else "http"
+    endpoint_url = f"{schema}://{HOST}:{PORT}/"
     return boto3.client(
         service_name="s3",
         aws_access_key_id=ACCESS_KEY,
@@ -67,7 +81,7 @@ def bucket(boto_client) -> str:
     :return: bucket name str
     """
     # yield "inbox"
-    bucket_name = "test_object_clerk"
+    bucket_name = "test-object-clerk"
     # Create Test Bucket
     try:
         boto_client.create_bucket(Bucket=bucket_name)
@@ -88,7 +102,7 @@ def empty_bucket(boto_client) -> str:
     :param boto_client: boto3.client instance
     :return: bucket name str
     """
-    bucket_name = "empty_bucket"
+    bucket_name = "empty-bucket"
     # Create Empty Bucket
     try:
         boto_client.create_bucket(Bucket=bucket_name)
@@ -109,7 +123,7 @@ def move_bucket(boto_client) -> str:
     :param boto_client: boto3.client instance
     :return: bucket name str
     """
-    bucket_name = "test_object_clerk_move"
+    bucket_name = "test-object-clerk-move"
     # Create Test Bucket
     try:
         boto_client.create_bucket(Bucket=bucket_name)
@@ -132,16 +146,16 @@ def object_store_data(boto_client, bucket) -> None:
     :param bucket: str name of the bucket to load the data in
     :return:
     """
-    with open("object_clerk/tests/example_object", mode="rb") as f:
+    with open(get_abs_path("example_object"), mode="rb") as f:
         # Add Get file
         boto_client.upload_fileobj(f, Bucket=bucket, Key="get_file")
-    with open("object_clerk/tests/example_object", mode="rb") as f:
+    with open(get_abs_path("example_object"), mode="rb") as f:
         # Add Delete file
         boto_client.upload_fileobj(f, Bucket=bucket, Key="delete_file")
-    with open("object_clerk/tests/example_object", mode="rb") as f:
+    with open(get_abs_path("example_object"), mode="rb") as f:
         # Add Copy File
         boto_client.upload_fileobj(f, Bucket=bucket, Key="copy_file")
-    with open("object_clerk/tests/example_object", mode="rb") as f:
+    with open(get_abs_path("example_object"), mode="rb") as f:
         # Add Copy File
         boto_client.upload_fileobj(f, Bucket=bucket, Key="move_file")
     yield
@@ -204,6 +218,23 @@ def object_clerk() -> ObjectClerk:
     return ObjectClerk(host=HOST, port=PORT, access_key=ACCESS_KEY, secret_key=SECRET_KEY, **RETRY)
 
 
+# stub for future testing
+@pytest.fixture(scope="session")
+def archive_clerk() -> ArchiveClerk:
+    """
+    Fixture providing a configured instance of the Archive clerk
+    :return: ArchiveClerk instance
+    """
+    return ArchiveClerk(
+        host=HOST,
+        port=PORT,
+        access_key=ACCESS_KEY,
+        secret_key=SECRET_KEY,
+        use_ssl=ARCHIVE_STORE_USE_SSL,
+        **RETRY,
+    )
+
+
 """
 Test Cases
 """
@@ -283,7 +314,7 @@ def test_get_object_info(object_clerk, object_store_data, bucket):
     :param bucket: Bucket containing the expected objects
     :return: None
     """
-    with open("object_clerk/tests/example_object", mode="rb") as f:
+    with open(get_abs_path("example_object"), mode="rb") as f:
         checksum = md5(f.read()).hexdigest()
     # When
     response = object_clerk.get_object_info(bucket, "get_file")
@@ -365,8 +396,8 @@ def test_delete_object(object_clerk, object_store_data, bucket):
 @pytest.mark.parametrize(
     "file",
     [
-        pytest.param("object_clerk/tests/example_object", id="str"),
-        pytest.param(Path("object_clerk/tests/example_object"), id="Path"),
+        pytest.param(get_abs_path("example_object"), id="str"),
+        pytest.param(Path(get_abs_path("example_object")), id="Path"),
         pytest.param(
             BufferedReader(BytesIO(b"EXAMPLE OBJECT FOR TESTING OBJECT CLERK OPERATIONS\n")),
             id="BufferedReader",
@@ -387,7 +418,7 @@ def test_upload_object(object_clerk, bucket, file):
     :param file: File to upload
     :return: None
     """
-    with open("object_clerk/tests/example_object", mode="rb") as f:
+    with open(get_abs_path("example_object"), mode="rb") as f:
         checksum = md5(f.read()).hexdigest()
     # When
     object_clerk.upload_object(file, bucket, "upload_file")
@@ -440,6 +471,31 @@ def test_upload_object_checksums_by_size(object_clerk, local_file, bucket):
     assert etag == checksum
 
 
+@pytest.mark.development
+def test_upload_object_metadata(object_clerk, local_file, bucket):
+    """
+    Given: ObjectClerk instance, object to upload
+    When: Uploading the object with extra header parameters
+    Then: The headers can be read and parsed
+    :param object_clerk: ObjectClerk instance
+    :param local_file: Fixture providing a file and its size
+    :param bucket: Bucket to load the object into
+    :return: None
+    """
+    file_path, _ = local_file
+    object_key = f"object_clerk/test/{uuid4().hex}.file"
+    # When
+    dataset_id = uuid4().hex
+    metadata = {"groupname": "DATASET", "groupid": dataset_id, "objecttype": "FILE"}
+    object_clerk.upload_object(file_path, bucket, object_key, metadata=metadata)
+    file_headers = object_clerk.get_object_info(bucket=bucket, object_key=object_key)
+    assert file_headers["x-amz-meta-groupid"] == dataset_id
+    assert file_headers["x-amz-meta-groupname"] == "DATASET"
+    assert file_headers["x-amz-meta-objecttype"] == "FILE"
+    # clean up
+    object_clerk.delete_object(bucket, object_key)
+
+
 @pytest.mark.development
 def test_upload_empty_file(object_clerk, bucket):
     """
@@ -492,10 +548,12 @@ def test_list_objects_not_truncated(object_clerk, bucket, object_store_data):
     """
     contents, is_truncated = object_clerk.list_objects(bucket)
 
-    assert len(contents) == 2
-    assert contents[0]["Key"] == "copy_file"
-    assert contents[1]["Key"] == "get_file"
+    # because object_clerk is session scoped, can be from 2 to 4 objects, depending on the order that tests are run
+    assert len(contents) >= 2
     assert not is_truncated
+    object_ids = [content["Key"] for content in contents]
+    assert "copy_file" in object_ids
+    assert "get_file" in object_ids
 
 
 @pytest.mark.development
